
/*******************************************************************************
* Copyright (c) 2017 Olivier Langella <olivier.langella@u-psud.fr>.
*
* This file is part of XTPcpp.
*
*     XTPcpp is free software: you can redistribute it and/or modify
*     it under the terms of the GNU General Public License as published by
*     the Free Software Foundation, either version 3 of the License, or
*     (at your option) any later version.
*
*     XTPcpp is distributed in the hope that it will be useful,
*     but WITHOUT ANY WARRANTY; without even the implied warranty of
*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
*
* Contributors:
*     Olivier Langella <olivier.langella@u-psud.fr> - initial API and implementation
******************************************************************************/

#include "peptidetablemodel.h"

#include <QDebug>
#include <QColor>
#include <pappsomspp/grouping/grppeptide.h>
#include "../../core/labeling/label.h"


PeptideTableModel::PeptideTableModel(QObject *parent)
    :QAbstractTableModel(parent)
{

    //ui->tableView->show();
    // QModelIndex topLeft = createIndex(0,0);
    //emit a signal to make the view reread identified data
    //emit dataChanged(topLeft, topLeft);
}


void PeptideTableModel::setProteinMatch(ProteinMatch * p_protein_match) {
    qDebug() << "PeptideTableModel::setProteinMatch begin " ;

    beginResetModel();
    _p_protein_match = p_protein_match;
    endResetModel();
    //QModelIndex topLeft = createIndex(0,0);
    //QModelIndex bottomRight = createIndex(rowCount(),columnCount());


    //emit dataChanged(topLeft, topLeft);
    //emit headerDataChanged(Qt::Horizontal, 0,3);
    qDebug() << "PeptideTableModel::setProteinMatch end " ;
}

ProteinMatch * PeptideTableModel::getProteinMatch() {
    return _p_protein_match;
}
int PeptideTableModel::rowCount(const QModelIndex &parent ) const {
    if (_p_protein_match != nullptr) {
        //qDebug() << "PeptideTableModel::rowCount(const QModelIndex &parent ) " << _p_protein_match->getPeptideMatchList().size();
        return (int) _p_protein_match->getPeptideMatchList().size();
    }
    return 0;
}
int PeptideTableModel::columnCount(const QModelIndex &parent ) const {
    return 17;
}
QVariant PeptideTableModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Horizontal) {
        switch (role) {
        case Qt::DisplayRole :
            return QVariant(getTitle(section));
            break;
        case Qt::ToolTipRole :
            return QVariant(getDescription(section));
            break;

        }
    }
//if (_p_peptide_list_window->getProjectWindow()->getProjectP()->getLabelingMethodSp().get() == nullptr) {
    return QVariant();
}
QVariant PeptideTableModel::data(const QModelIndex &index, int role ) const {
    // generate a log message when this method gets called
    int row = index.row();
    int col = index.column();
    //qDebug() << QString("row %1, col%2, role %3")
    //         .arg(row).arg(col).arg(role);

    switch(role) {
    case Qt::CheckStateRole:

        if (col == 0) //add a checkbox to cell(1,0)
        {
            if ( _p_protein_match->getPeptideMatchList().at(row)->isChecked()) {
                return Qt::Checked;
            }
            else {
                return Qt::Unchecked;
            }
        }
        break;
    case Qt::BackgroundRole:
        if (_p_protein_match->getPeptideMatchList().at(row)->isValid() == false)
        {
            return QVariant(QColor("grey"));
        }
        break;
    case Qt::DisplayRole:
        if (_p_protein_match == nullptr) {
        }
        else {
            pappso::GrpPeptide * p_grp_peptide;
            const Label * p_label;
            GroupingGroup * p_grp;
            switch (col) {
            case (std::int8_t)PeptideListColumn::checked:
                return QVariant();
                break;

            case (std::int8_t)PeptideListColumn::peptide_grouping_id:
                p_grp_peptide = _p_protein_match->getPeptideMatchList().at(row)->getGrpPeptideSp().get();
                if (p_grp_peptide != nullptr) return p_grp_peptide->getGroupingId();
                return QVariant();
                break;
            case (std::int8_t)PeptideListColumn::sample:
                return _p_protein_match->getPeptideMatchList().at(row)->getIdentificationDataSource()->getSampleName();
                break;
            case (std::int8_t)PeptideListColumn::scan:
                return QVariant ((quint32) _p_protein_match->getPeptideMatchList().at(row)->getScan());
                break;
            case (std::int8_t)PeptideListColumn::rt:
                return QVariant ((qreal) _p_protein_match->getPeptideMatchList().at(row)->getRetentionTime());
                break;
            case (std::int8_t)PeptideListColumn::charge:
                return QVariant ((qreal) _p_protein_match->getPeptideMatchList().at(row)->getCharge());
                break;
            case (std::int8_t)PeptideListColumn::sequence:
                return _p_protein_match->getPeptideMatchList().at(row)->getPeptideXtpSp().get()->getSequence();
                break;
            case (std::int8_t)PeptideListColumn::modifs:
                return _p_protein_match->getPeptideMatchList().at(row)->getPeptideXtpSp().get()->getModifString();
                break;
            case (std::int8_t)PeptideListColumn::label:
                p_label = _p_protein_match->getPeptideMatchList().at(row)->getPeptideXtpSp().get()->getLabel();
                if (p_label != nullptr) {
                    return QVariant(p_label->getXmlId());
                }
                else {
                    return QVariant();
                }
                break;
            case (std::int8_t)PeptideListColumn::start:
                return QVariant((qreal) _p_protein_match->getPeptideMatchList().at(row)->getStart()+1);
                break;
            case (std::int8_t)PeptideListColumn::length:
                return QVariant((qreal) _p_protein_match->getPeptideMatchList().at(row)->getPeptideXtpSp().get()->size());
                break;
            case (std::int8_t)PeptideListColumn::used:
                p_grp = _p_protein_match->getGroupingGroupSp().get();
                if (p_grp != nullptr) return QVariant((qreal)  p_grp->countSubgroupPresence(_p_protein_match->getPeptideMatchList().at(row)));
                return QVariant();
                break;
            case (std::int8_t)PeptideListColumn::subgroups:
                p_grp = _p_protein_match->getGroupingGroupSp().get();
                if (p_grp != nullptr) return QVariant( p_grp->getSubgroupIdList(_p_protein_match->getPeptideMatchList().at(row)).join(" "));
                return QVariant();
                break;
            case (std::int8_t)PeptideListColumn::Evalue:
                return QVariant((qreal) _p_protein_match->getPeptideMatchList().at(row)->getEvalue());
                break;
            case (std::int8_t)PeptideListColumn::theoretical_mhplus:
                return QVariant((qreal) _p_protein_match->getPeptideMatchList().at(row)->getPeptideXtpSp().get()->getMz(1));
                break;
            case (std::int8_t)PeptideListColumn::delta_mhplus:
                return QVariant((qreal) _p_protein_match->getPeptideMatchList().at(row)->getDeltaMass());
                break;
            case (std::int8_t)PeptideListColumn::hyperscore:
                return _p_protein_match->getPeptideMatchList().at(row)->getParam(PeptideMatchParam::tandem_hyperscore);
                break;
            }
        }
        return QString("Row%1, Column%2")
               .arg(index.row() + 1)
               .arg(index.column() +1);
    }
    return QVariant();
}

void PeptideTableModel::onTableClicked(const QModelIndex &index)
{
    int row = index.row();
    int col = index.column();
    if (col == 0) //add a checkbox to cell(1,0)
    {
        if ( _p_protein_match->getPeptideMatchList().at(row)->isChecked()) {
            _p_protein_match->getPeptideMatchList().at(row)->setChecked(false);
        }
        else {
            _p_protein_match->getPeptideMatchList().at(row)->setChecked(true);
        }
        //emit dataChanged(index, index);
    }
    else {
        emit peptideMatchClicked(_p_protein_match->getPeptideMatchList().at(row));
    }
}

void PeptideTableModel::onPeptideDataChanged() {
    qDebug() << "PeptideTableModel::onPeptideDataChanged begin " << rowCount();
    emit layoutAboutToBeChanged();
    emit layoutChanged();
}



PeptideListColumn PeptideTableModel::getPeptideListColumn(std::int8_t column){
    return static_cast<PeptideListColumn>(column);
}
const QString PeptideTableModel::getTitle(PeptideListColumn column) {
    qDebug() << "PeptideTableModel::getTitle begin ";
    return PeptideTableModel::getTitle((std::int8_t) column);
    //qDebug() << "ProteinTableModel::getTitle end ";
}
const QString PeptideTableModel::getDescription(PeptideListColumn column) {
    qDebug() << "PeptideTableModel::columnCount begin ";
    return PeptideTableModel::getDescription((std::int8_t) column);
    //qDebug() << "ProteinTableModel::columnCount end ";
}

const QString PeptideTableModel::getTitle(std::int8_t column) {

    qDebug() << "PeptideTableModel::getTitle begin " << column;
    switch (column) {

    case (std::int8_t) PeptideListColumn::checked:
        return "checked";
        break;
    case (std::int8_t) PeptideListColumn::peptide_grouping_id:
        return "peptide ID";
        break;
    case (std::int8_t) PeptideListColumn::sample:
        return "sample";
        break;
    case (std::int8_t) PeptideListColumn::scan:
        return "scan";
        break;
    case (std::int8_t) PeptideListColumn::rt:
        return "RT";
        break;
    case (std::int8_t) PeptideListColumn::charge:
        return "charge";
        break;
    case (std::int8_t) PeptideListColumn::sequence:
        return "sequence";
        break;
    case (std::int8_t) PeptideListColumn::modifs:
        return "modifs";
        break;
    case (std::int8_t) PeptideListColumn::label:
        return "label";
        break;
    case (std::int8_t) PeptideListColumn::start:
        return "start";
        break;
    case (std::int8_t) PeptideListColumn::length:
        return "length";
        break;
    case (std::int8_t) PeptideListColumn::used:
        return "used";
        break;
    case (std::int8_t) PeptideListColumn::subgroups:
        return "subgroups";
        break;
    case (std::int8_t) PeptideListColumn::Evalue:
        return "Evalue";
        break;
    case (std::int8_t) PeptideListColumn::theoretical_mhplus:
        return "theoretical MH+";
        break;
    case (std::int8_t) PeptideListColumn::delta_mhplus:
        return "delta MH+";
        break;
    case (std::int8_t) PeptideListColumn::hyperscore:
        return "hyperscore";
    }
    return "";
}

const QString PeptideTableModel::getDescription(std::int8_t column) {

    qDebug() << "ProteinTableModel::getDescription begin " << column;
    switch (column) {

    case (std::int8_t) PeptideListColumn::checked:
        return "manual protein check";
        break;

    case (std::int8_t) PeptideListColumn::peptide_grouping_id:
        return "unique peptide identifier within this grouping experiment";
        break;
    case (std::int8_t) PeptideListColumn::sample:
        return "MS sample name";
        break;
    case (std::int8_t) PeptideListColumn::scan:
        return "scan number";
        break;
    case (std::int8_t) PeptideListColumn::rt:
        return "retention time in seconds";
        break;
    case (std::int8_t) PeptideListColumn::charge:
        return "peptide charge";
        break;
    case (std::int8_t) PeptideListColumn::sequence:
        return "peptide sequence";
        break;
    case (std::int8_t) PeptideListColumn::modifs:
        return "peptide modifications";
        break;
    case (std::int8_t) PeptideListColumn::label:
        return "peptide label";
        break;
    case (std::int8_t) PeptideListColumn::start:
        return "peptide start position on protein";
        break;
    case (std::int8_t) PeptideListColumn::length:
        return "peptide length";
        break;
    case (std::int8_t) PeptideListColumn::used:
        return "number of subgroups in which this peptide is used";
        break;
    case (std::int8_t) PeptideListColumn::subgroups:
        return "list of subgroups in which this peptide is used";
        break;
    case (std::int8_t) PeptideListColumn::Evalue:
        return "peptide Evalue";
        break;
    case (std::int8_t) PeptideListColumn::theoretical_mhplus:
        return "peptide theoretical MH+";
        break;
    case (std::int8_t) PeptideListColumn::delta_mhplus:
        return "peptide mass difference betwenn observed mass and theoretical mass";
        break;
    case (std::int8_t) PeptideListColumn::hyperscore:
        return "X!Tandem hyperscore";

    }
    return "";
}
